-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

---------------------
-- Prep the module --
---------------------
local M = {}
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 11000

-----------------
-- Debug stuff --
-----------------
local ugfxCalled = 0
local speedDipped = 0

---------------------------
-- Tuning menu variables --
---------------------------

-- Button momentary or toggled
local buttonToggle = 1

-- Particle emission rate
local steamFPS = 1 / 30

-- Particle downward speed
local downSpeed = 5

---------------------
-- State variables --
---------------------

-- Elapsed launch time as of right now
local steamTime = 0

-- Last trigger states
local lastTrig = 0

-- Particle emission source and goal nodes
local steamEnabled = 0
local sourceNode = nil
local goalNode = nil

------------------------------------
-- Determine edges on triggers    --
------------------------------------
-- Detected edges
-- 0 = No edges
-- 1 = Rising edge
-- 2 = Falling edge
local function getTriggerEdges()
	local	edgeDir = 0
	
	-- Trigger activated
	local steamTrigger = (electrics.values['hb9kSteamTrigger'] or 0)
	
	-- Detect forward edge
	if steamTrigger == 1 then if lastTrig == 0 then edgeDir = 1 end
	else                      if lastTrig == 1 then edgeDir = 2 end end
	
	-- New trigger states have become old trigger states
	lastTrig = steamTrigger
	
	-- Return result
	return edgeDir
end

---------------------------------------------
-- Graphics update function                --
-- Updates other stuff too according to dt --
---------------------------------------------
local function updateGFX(dt)
	
	-- Got trigger?
	local edgeDir
	edgeDir = getTriggerEdges()
	
	-- Retrieve some electrics
	local steamOn = (electrics.values['hb9kSteamOn'] or 0)
	
	----------------------------
	-- Momentary trigger mode --
	----------------------------
	if buttonToggle <= 0.5 then
		
		-- Steam on; Switch it off
		if steamOn == 1 then if edgeDir == 2 then electrics.values['hb9kSteamOn'] = 0 end
			
		-- Steam off; Switch it on
		else                 if edgeDir == 1 then electrics.values['hb9kSteamOn'] = 1 end end
		
	--------------------------
	-- Toggled trigger mode --
	--------------------------
	else
		
		-- Steam on; Switch it off
		if steamOn == 1 then if edgeDir == 1 then electrics.values['hb9kSteamOn'] = 0 end
			
		-- Steam off; Switch it on
		else                 if edgeDir == 1 then electrics.values['hb9kSteamOn'] = 1 end end
	end
	
	----------------
	-- Emit steam --
	----------------
	if electrics.values['hb9kSteamOn'] == 1 and steamEnabled == 1 then
		if steamOn == 0 then
			steamTime = 0
		end
		if steamTime >= steamFPS then
			steamTime = 0
		--  obj:addParticleByNodesRelative(nodeId1,    nodeid2,  velocity,  type, width, count)
			obj:addParticleByNodesRelative(sourceNode, goalNode, -downSpeed, 24,   0.1,   20)
		end
		steamTime = steamTime + dt
	end
	
	-- Debug messages
	if ugfxCalled == 0 then
		ugfxCalled = 1
		log("D", "Steamy Nose Gear", "updateGFX function called")
	end
	if dt >= (2/30) and speedDipped == 0 then
		speedDipped = 1
		log("D", "Steamy Nose Gear", "Speed dipped below 15 FPS")
	end
end

-- Reset what we need to
local function reset()
	electrics.values['hb9kSteamOn'] = 0
	steamTime = 0
	speedDipped = 0
	lastTrig = 0
	log("D", "Steamy Nose Gear", "reset function called")
end

-- Load steam emitter nodes
local function initSteamNodes()
	sourceNode = nil
	goalNode = nil
	
	if v.data.activeParts.me262_gear_n_steamy ~= nil then
		for k, node in pairs (v.data.nodes) do
			if node.name == "gear_nl_1"  then sourceNode = k elseif
			   node.name == "gear_n_1" then goalNode = k end
		end
	elseif v.data.activeParts.an12_gear_F_steamy ~= nil then
		for k, node in pairs (v.data.nodes) do
			if node.name == "gfwc"  then sourceNode = k elseif
			   node.name == "gfbf" then goalNode = k end
		end
	else
		log("E", "Steamy Nose Gear", "Steamy Nose Gear part not found!")
	end
	
	if sourceNode ~= nil and goalNode ~= nil then
		steamEnabled = 1
	else
		steamEnabled = 0
		log("E", "Steamy Nose Gear", "Steam NOT enabled!")
	end
end

-- Initialize everything
local function init(jbeamdata)
	buttonToggle = (v.data.variables["$hb9k_GearSteamToggle"].val or 0)
	downSpeed = (v.data.variables["$hb9k_GearSteamSpeed"].val or 0)
	steamFPS = 1 / 30
	initSteamNodes()
	log("D", "Steamy Nose Gear", "init function called")
end

-- Public interface
M.init = init
M.reset = reset
M.updateGFX = updateGFX

-- Return the module
return M
